# MUMPS Installation
MESTI.jl uses the parallel version of MUMPS for the augmented partial factorization (APF) method, and optionally for the factorize-and-solve method. Here are the steps to install MUMPS.

## Download MUMPS
Go to the [MUMPS website](https://mumps-solver.org/index.php?page=dwnld) and fill out the download request form. The MUMPS maintainers will email you the download link.

## Prerequisites
To compile the parallel version of MUMPS, you need compilation tools like <code>make</code>, <code>ar</code>, and <code>ranlib</code>, C and Fortran compilers, BLAS library, LAPACK library, ScaLAPACK library, and MPI library. Instructions specific to the operating system are provided below:
1. [linux](./linux)
2. [macOS](./macOS)
3. [windows](./windows)

If you are interested in 3D systems or memory usage in 2D systems is important for you, we highly recommend you install the [METIS](http://glaros.dtc.umn.edu/gkhome/metis/metis/overview) (version 5.1.0) program for graph partitioning (not to be confused with MESTI). We can install METIS in the following steps:
(a) Downloading METIS (version 5.1.0)
```shell
wget http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/metis-5.1.0.tar.gz
```
(b) Decompress metis-5.1.0.tar.gz
```shell
tar zxvf metis-5.1.0.tar.gz
```
(c) Setting METIS to double precision
```shell
sed -i "43s/32/64/" metis-5.1.0/include/metis.h
```
(d) Installing METIS
```shell
cd metis-5.1.0; make config; sudo make install;
```
Then, by default, the library file, header file, and binaries will be installed in `/usr/local/lib`, `/usr/local/include`, and `/usr/local/bin`. In some rare cases, your machine cannot find METIS libraries by itself when you run Julia interface for MUMPS. You can append the METIS libraries to your `LD_LIBRARYP_PATH`
```shell
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LMETISDIR
```

`LMETISDIR` is the path to the folder where the METIS library is.

After installing METIS, if you set <code>opts.use_METIS = true</code> in <code>mesti()</code> or <code>mesti2s()</code>, MUMPS will use METIS for matrix ordering. From our experience, in 2D, AMD is usually faster when using the APF method, but METIS can sometimes reduce memory usage. In 3D, METIS is strongly recommended, which is much faster than AMD. By default, 2D systems use AMD, while 3D systems use METIS (if it is available).

## Compile MUMPS
Suppose you downloaded the 5.6.2 version of MUMPS to your ~/Downloads/ folder. Then, go to the folder where you want to compile MUMPS, and enter
```shell
tar zxvf ~/Downloads/MUMPS_5.6.2.tar.gz
cd MUMPS_5.6.2
```
in terminal.

Read the file <code>INSTALL</code>, copy the closest <code>Makefile.inc</code> file in the <code>Make.inc</code> folder, and modify it to fit your environment and machine. Most importantly, in <code>Makefile.inc</code> you need to specify:
 - <code>CC</code>: the C compiler to use
 - <code>FC</code>: the Fortran compiler to use
 - <code>FL</code>: the Fortran linker to use
 - <code>LAPACK</code>: how the Fortran compiler can link to the LAPACK library
 - <code>SCALAP</code>: how the Fortran compiler can link to the ScaLAPACK library
 - <code>LIBBLAS</code>: how the Fortran compiler can link to the BLAS library
 - <code>RPATH_OPT</code>: the path to shared libraries that will be built up, such as <code>.../MUMPS_5.6.2/lib/</code>
where <code>...</code> is the path to MUMPS_5.6.2 folder. 

Note that from our experience, <code>RPATH_OPT</code> must be specified to successfully install the parallel version of MUMPS on Linux and Windows.

If you installed METIS, you also need to specify
 - <code>LMETISDIR</code>: path to the folder where the METIS library is
 - <code>IMETIS</code>: path to the folder where the METIS headers are

and add <code>-Dmetis</code> to <code>ORDERINGSF</code>.

Examples of <code>Makefile.inc</code> are provided below:
1. [linux](./linux/Makefile.inc)
2. [macOS](./macOS/Makefile.inc)
3. [windows](./windows/Makefile.inc)

To download, click the link above, click on the "Raw" button, and right-click to save the file. If the browser adds a .txt file extension, rename it to remove the txt extension.

After done with <code>Makefile.inc</code>, enter
```shell
make allshared
```
in the terminal, which will compile the parallel version of MUMPS with single and double precision for real and complex variables (*i.e.*, <code>smumps</code>, <code>dmumps</code>, <code>cmumps</code>, <code>zmumps</code>).

If there is no error, check if the following files have been generated in the <code>lib</code> folder: <code>libsmumps.so</code>, <code>libdmumps.so</code>, <code>libcmumps.so</code>, and <code>libzmumps.so</code>.

Warning messages from the Fortran compiler are normal and can be ignored.

If there is an error, read the message and try to figure out where it comes from and/or look it up and address it. Before recompiling with <code>make allshared</code>, be sure to type <code>make clean</code> first to remove files generated by the previous compile attempt.

## Running MUMPS in Julia
After compiling the parallel version of MUMPS, in <code>startup.jl</code> we should set the Julia environment variable <code>MUMPS_PREFIX</code>, which is the path to your own MUMPS libraries through the terminal: 

```shell
mkdir ~/.julia/config
echo 'ENV["MUMPS_PREFIX"] = ".../MUMPS_5.6.2/lib"' >> ~/.julia/config/startup.jl
```

where <code>...</code> is the path to MUMPS_5.6.2 folder.


When we run Julia interface for MUMPS, the machine may not find the libraries by itself. To solve those issues, please follow the steps depending on your OS:

- Linux or Windows

We can append those library paths to `LD_PRELOAD` before running Julia. For example, with the Intel oneAPI installed under /opt, we can type,

```shell
source /opt/intel/oneapi/mkl/latest/env/vars.sh
source /opt/intel/oneapi/mpi/latest/env/vars.sh
source /opt/intel/oneapi/compiler/latest/env/vars.sh
export LD_PRELOAD=$LD_PRELOAD:$MKLROOT/lib/intel64/libmkl_intel_lp64.so
export LD_PRELOAD=$LD_PRELOAD:$MKLROOT/lib/intel64/libmkl_intel_thread.so
export LD_PRELOAD=$LD_PRELOAD:/opt/intel/oneapi/inspector/latest/lib64/libiomp5.so
export LD_PRELOAD=$LD_PRELOAD:$MKLROOT/lib/intel64/libmkl_core.so
export LD_PRELOAD=$LD_PRELOAD:$MKLROOT/lib/intel64/libmkl_blacs_intelmpi_lp64.so
export LD_PRELOAD=$LD_PRELOAD:$MKLROOT/lib/intel64/libmkl_scalapack_lp64.so
```

or

- macOS

We can configure [MPI.jl](https://juliaparallel.org/MPI.jl/stable/configuration/) before running MUMPS in Julia. The steps are straightforward using MPIPreferences.jl. First, install MPIPreferences.jl by entering
```
julia --project -e 'using Pkg; Pkg.add("MPIPreferences")'
```
in terminal. Then run <code>MPIPreferences.use_system_binary()</code> in Julia or through the command line:
```
julia --project -e 'using MPIPreferences; MPIPreferences.use_system_binary()'
```
This should automatically find the OpenMPI installed in macOS.

Now, you are ready to install MESTI.jl. Please go back to [install MESTI.jl](../#installation).